@@ -1,23 +1,49 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """Parser."""
-from .. import config
 import re
 
+from mriqc import config
+
 
 def _build_parser():
     """Build parser object."""
     import sys
-    from shutil import which
+    import warnings
+    from argparse import Action, ArgumentDefaultsHelpFormatter, ArgumentParser
     from functools import partial
     from pathlib import Path
-    from argparse import (
-        ArgumentParser,
-        ArgumentDefaultsHelpFormatter,
-    )
+    from shutil import which
+
     from packaging.version import Version
+
     from .version import check_latest, is_flagged
 
+    class DeprecateAction(Action):
+        def __call__(self, parser, namespace, values, option_string=None):
+            warnings.warn(f"Argument {option_string} is deprecated and is *ignored*.")
+            delattr(namespace, self.dest)
+
     def _path_exists(path, parser):
         """Ensure a given path exists."""
         if path is None or not Path(path).exists():
@@ -32,9 +58,10 @@ def _build_parser():
         return value
 
     def _to_gb(value):
-        scale = {"G": 1, "T": 10 ** 3, "M": 1e-3, "K": 1e-6, "B": 1e-9}
+        scale = {"G": 1, "T": 10**3, "M": 1e-3, "K": 1e-6, "B": 1e-9}
         digits = "".join([c for c in value if c.isdigit()])
-        units = value[len(digits):] or "G"
+        n_digits = len(digits)
+        units = value[n_digits:] or "G"
         return int(digits) * scale[units[0]]
 
     def _drop_sub(value):
@@ -77,7 +104,7 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         "bids_dir",
         action="store",
         type=PathExists,
-        help="the root folder of a BIDS valid dataset (sub-XXXXX folders should "
+        help="The root folder of a BIDS valid dataset (sub-XXXXX folders should "
         "be found at the top level in this folder).",
     )
     parser.add_argument(
@@ -86,7 +113,7 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         type=Path,
         help="The directory where the output files "
         "should be stored. If you are running group level analysis "
-        "this folder should be prepopulated with the results of the"
+        "this folder should be prepopulated with the results of the "
         "participant level analysis.",
     )
     parser.add_argument(
@@ -107,7 +134,17 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         dest="verbose_count",
         action="count",
         default=0,
-        help="increases log verbosity for each occurrence, debug level is -vvv",
+        help="Increases log verbosity for each occurrence, debug level is -vvv.",
+    )
+
+    # TODO: add 'mouse', 'macaque', and other populations once the pipeline is working
+    parser.add_argument(
+        "--species",
+        action="store",
+        type=str,
+        default="human",
+        choices=["human", "rat"],
+        help="Use appropriate template for population",
     )
 
     g_bids = parser.add_argument_group("Options for filtering BIDS queries")
@@ -117,38 +154,45 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         action="store",
         nargs="+",
         type=_drop_sub,
-        help="a space delimited list of participant identifiers or a single "
-        "identifier (the sub- prefix can be removed)",
+        help="A space delimited list of participant identifiers or a single "
+        "identifier (the sub- prefix can be removed).",
     )
     g_bids.add_argument(
         "--session-id",
         action="store",
         nargs="*",
         type=str,
-        help="filter input dataset by session id",
+        help="Filter input dataset by session ID.",
     )
     g_bids.add_argument(
         "--run-id",
         action="store",
         type=int,
         nargs="*",
-        help="filter input dataset by run id (only integer run ids are valid)",
+        help="Filter input dataset by run ID (only integer run IDs are valid).",
     )
     g_bids.add_argument(
         "--task-id",
         action="store",
         nargs="*",
         type=str,
-        help="filter input dataset by task id",
+        help="Filter input dataset by task ID.",
     )
     g_bids.add_argument(
         "-m",
         "--modalities",
         action="store",
         nargs="*",
-        help="filter input dataset by MRI type",
+        help="Filter input dataset by MRI type.",
+    )
+    g_bids.add_argument("--dsname", type=str, help="A dataset name.")
+    g_bids.add_argument(
+        "--bids-database-dir",
+        metavar="PATH",
+        type=PathExists,
+        help="Path to an existing PyBIDS database folder, for faster indexing "
+        "(especially useful for large datasets).",
     )
-    g_bids.add_argument("--dsname", type=str, help="a dataset name")
 
     # General performance
     g_perfm = parser.add_argument_group("Options to handle performance")
@@ -159,14 +203,29 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         "-n-cpus",
         action="store",
         type=PositiveInt,
-        help="maximum number of threads across all processes",
+        help="""\
+Maximum number of simultaneously running parallel processes executed by *MRIQC* \
+(e.g., several instances of ANTs' registration). \
+However, when ``--nprocs`` is greater or equal to the ``--omp-nthreads`` option, \
+it also sets the maximum number of threads that simultaneously running processes \
+may aggregate (meaning, with ``--nprocs 16 --omp-nthreads 8`` a maximum of two \
+8-CPU-threaded processes will be running at a given time). \
+Under this mode of operation, ``--nprocs`` sets the maximum number of processors \
+that can be assigned work within an *MRIQC* job, which includes all the processors \
+used by currently running single- and multi-threaded processes. \
+If ``None``, the number of CPUs available will be automatically assigned (which may \
+not be what you want in, e.g., shared systems like a HPC cluster.""",
     )
     g_perfm.add_argument(
         "--omp-nthreads",
         "--ants-nthreads",
         action="store",
         type=PositiveInt,
-        help="maximum number of threads per-process",
+        help="""\
+Maximum number of threads that multi-threaded processes executed by *MRIQC* \
+(e.g., ANTs' registration) can use. \
+If ``None``, the number of CPUs available will be automatically assigned (which may \
+not be what you want in, e.g., shared systems like a HPC cluster.""",
     )
     g_perfm.add_argument(
         "--mem",
@@ -175,14 +234,14 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         dest="memory_gb",
         action="store",
         type=_to_gb,
-        help="upper bound memory limit for MRIQC processes",
+        help="Upper bound memory limit for MRIQC processes.",
     )
     g_perfm.add_argument(
         "--testing",
         dest="debug",
         action="store_true",
         default=False,
-        help="use testing settings for a minimal footprint",
+        help="Use testing settings for a minimal footprint.",
     )
     g_perfm.add_argument(
         "-f",
@@ -190,14 +249,14 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         action="store_true",
         default=True,
         help="Cast the input data to float32 if it's represented in higher precision "
-        "(saves space and improves perfomance)",
+        "(saves space and improves perfomance).",
     )
     g_perfm.add_argument(
         "--pdb",
         dest="pdb",
         action="store_true",
         default=False,
-        help="open Python debugger (pdb) on exceptions",
+        help="Open Python debugger (pdb) on exceptions.",
     )
 
     # Control instruments
@@ -208,7 +267,7 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         action="store",
         type=Path,
         default=Path("work").absolute(),
-        help="path where intermediate results should be stored",
+        help="Path where intermediate results should be stored.",
     )
     g_outputs.add_argument("--verbose-reports", default=False, action="store_true")
     g_outputs.add_argument(
@@ -218,20 +277,25 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         help="Write workflow graph.",
     )
     g_outputs.add_argument(
-        "--dry-run", action="store_true", default=False, help="Do not run the workflow."
-    )
-    g_outputs.add_argument(
-        "--profile",
+        "--dry-run",
         action="store_true",
         default=False,
-        help="hook up the resource profiler callback to nipype",
+        help="Do not run the workflow.",
+    )
+    g_outputs.add_argument(
+        "--resource-monitor",
+        "--profile",
+        dest="resource_monitor",
+        action="store_true",
+        default=False,
+        help="Hook up the resource profiler callback to nipype.",
     )
     g_outputs.add_argument(
         "--use-plugin",
         action="store",
         default=None,
         type=Path,
-        help="nipype plugin configuration file",
+        help="Nipype plugin configuration file.",
     )
     g_outputs.add_argument(
         "--no-sub",
@@ -252,20 +316,20 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         "--webapi-url",
         action="store",
         type=str,
-        help="IP address where the MRIQC WebAPI is listening",
+        help="IP address where the MRIQC WebAPI is listening.",
     )
     g_outputs.add_argument(
         "--webapi-port",
         action="store",
         type=int,
-        help="port where the MRIQC WebAPI is listening",
+        help="Port where the MRIQC WebAPI is listening.",
     )
 
     g_outputs.add_argument(
         "--upload-strict",
         action="store_true",
         default=False,
-        help="upload will fail if if upload is strict",
+        help="Upload will fail if upload is strict.",
     )
 
     # ANTs options
@@ -274,12 +338,12 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         "--ants-float",
         action="store_true",
         default=False,
-        help="use float number precision on ANTs computations",
+        help="Use float number precision on ANTs computations.",
     )
     g_ants.add_argument(
         "--ants-settings",
         action="store",
-        help="path to JSON file with settings for ANTS",
+        help="Path to JSON file with settings for ANTs.",
     )
 
     # Functional workflow settings
@@ -290,7 +354,7 @@ Automated Quality Control and visual reports for Quality Assesment of structural
             action="store_true",
             default=False,
             help="Run ICA on the raw data and include the components "
-            "in the individual reports (slow but potentially very insightful)",
+            "in the individual reports (slow but potentially very insightful).",
         )
     g_func.add_argument(
         "--fft-spikes-detector",
@@ -303,41 +367,41 @@ Automated Quality Control and visual reports for Quality Assesment of structural
         action="store",
         default=0.2,
         type=float,
-        help="Threshold on Framewise Displacement estimates to detect outliers.",
+        help="Threshold on framewise displacement estimates to detect outliers.",
     )
     g_func.add_argument(
         "--deoblique",
         action="store_true",
         default=False,
-        help="Deoblique the functional scans during head motion "
-        "correction preprocessing",
+        help="Deoblique the functional scans during head motion correction "
+        "preprocessing.",
     )
     g_func.add_argument(
         "--despike",
         action="store_true",
         default=False,
         help="Despike the functional scans during head motion correction "
-        "preprocessing",
+        "preprocessing.",
     )
     g_func.add_argument(
         "--start-idx",
-        action="store",
+        action=DeprecateAction,
         type=int,
-        help="Initial volume in functional timeseries that should be "
-        "considered for preprocessing",
+        help="DEPRECATED Initial volume in functional timeseries that should be "
+        "considered for preprocessing.",
     )
     g_func.add_argument(
         "--stop-idx",
-        action="store",
+        action=DeprecateAction,
         type=int,
-        help="Final volume in functional timeseries that should be "
-        "considered for preprocessing",
+        help="DEPRECATED Final volume in functional timeseries that should be "
+        "considered for preprocessing.",
     )
     g_func.add_argument(
         "--correct-slice-timing",
         action="store_true",
         default=False,
-        help="Perform slice timing correction",
+        help="Perform slice timing correction.",
     )
 
     latest = check_latest()
@@ -366,13 +430,13 @@ discourage its usage.""",
 def parse_args(args=None, namespace=None):
     """Parse args and run further checks on the command line."""
     from logging import DEBUG
+
     from ..utils.bids import collect_bids_data
 
     parser = _build_parser()
     opts = parser.parse_args(args, namespace)
     config.execution.log_level = int(max(25 - 5 * opts.verbose_count, DEBUG))
     config.from_dict(vars(opts))
-    config.loggers.init()
 
     # Load base plugin_settings from file if --use-plugin
     if opts.use_plugin is not None:
@@ -388,17 +452,6 @@ def parse_args(args=None, namespace=None):
                 "nprocs", config.nipype.nprocs
             )
 
-    # Resource management options
-    # Note that we're making strong assumptions about valid plugin args
-    # This may need to be revisited if people try to use batch plugins
-    if 1 < config.nipype.nprocs < config.nipype.omp_nthreads:
-        config.loggers.cli.warning(
-            "Per-process threads (--omp-nthreads=%d) exceed total "
-            "threads (--nthreads/--n_cpus=%d)",
-            config.nipype.omp_nthreads,
-            config.nipype.nprocs,
-        )
-
     bids_dir = config.execution.bids_dir
     output_dir = config.execution.output_dir
     work_dir = config.execution.work_dir
@@ -500,13 +553,23 @@ Please, check out your currently set filters:
         [i for sublist in config.workflow.inputs.values() for i in sublist]
     )
 
+    # set specifics for alternative populations
+    if opts.species.lower() != "human":
+        config.workflow.species = opts.species
+        # TODO: add other species once rats are working
+        if opts.species.lower() == "rat":
+            config.workflow.template_id = "Fischer344"
+            config.workflow.headmask = "NoBET"
+            # block uploads for the moment; can be reversed before wider release
+            config.execution.no_sub = True
+
 
 def _get_biggest_file_size_gb(files):
     import os
 
     max_size = 0
     for file in files:
-        size = os.path.getsize(file) / (1024 ** 3)
+        size = os.path.getsize(file) / (1024**3)
         if size > max_size:
             max_size = size
     return max_size
